unit UFormSizePosistion;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.StdCtrls, Vcl.Forms, Vcl.Dialogs,Vcl.ExtCtrls,
  TypInfo, Registry, ColorButton;



const
   CRegKey = 'Software\Demos\WdwStateDemo\1.0';
  //  CRegKey = the actual registration in the Registry
Var
  OldWindowProc    : Pointer; {Variable for the old windows proc}
  MyMsg            : DWord;
type
  TfrmSizePosition = class(TForm)
    ClrBtnWindowSetting: TColorButton;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure ClrBtnWindowSettingClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmSizePosition: TfrmSizePosition;

implementation

{$R *.dfm}

function ReadIntFromReg(Reg: TRegistry; Name: string;
  Def: Integer): Integer;
  {Reads integer with given name from registry and returns it
  If no such value exists, returns Def default value}
begin
 if Reg.ValueExists(Name) then
    Result := Reg.ReadInteger(Name)
  else
    Result := Def;
end;

Function NewWindowProc(WindowHandle : hWnd;
                       TheMessage   : DWord ; //LongInt
                       ParamW       : LongInt;
                       ParamL       : LongInt) : LongInt Stdcall;
Begin

   If TheMessage = MyMsg  Then
   Begin
   {Tell the application to restore, let it restore the form}
   SendMessage(Application.handle, WM_SYSCOMMAND, SC_RESTORE, 0);
   SetForegroundWindow(Application.Handle);
   {We handled the message - we are done}
   Result := 0;
   exit;
   End;
   {Call the original winproc}
   Result := CallWindowProc(OldWindowProc,
                        WindowHandle,
                        TheMessage,
                        ParamW,
                        ParamL);
End;

procedure TfrmSizePosition.FormCreate(Sender: TObject);
begin
  OldWindowProc := Pointer(SetWindowLong(frmSizePosition.Handle,
                                       GWL_WNDPROC,
                                       LongInt(@NewWindowProc)));
end;

procedure TfrmSizePosition.ClrBtnWindowSettingClick(Sender: TObject);
var
  Reg: TRegistry;         // the registry
  State: Integer;         // state of wdw
  Pl : TWindowPlacement;  // used for API call
  R: TRect;               // used for wdw pos
begin
  {Calculate window's normal size and position using
  Windows API call - the form's Width, Height, Top and
  Left properties will give maximized window size if
  form is maximised, which is not what we want here}
  Pl.Length := SizeOf(TWindowPlacement);
  GetWindowPlacement(Self.Handle, @Pl);
  R := Pl.rcNormalPosition;
  Reg := TRegistry.Create;
  try
    // Open required key - and create it if it doesn't exist
    Reg.RootKey := HKEY_CURRENT_USER;
    Reg.OpenKey(CRegKey, True);
    // Write window size and position
    Reg.WriteInteger('Width', R.Right-R.Left);
    Reg.WriteInteger('Height', R.Bottom-R.Top);
    Reg.WriteInteger('Left', R.Left);
    Reg.WriteInteger('Top', R.Top);
    // Write out state of window
    {Record window state (maximised, minimised or normal)
    - special case when minimized since form window is simply
    hidden when minimised, and application window is actually
    the one minimised - so we check to see if application
    window *is* minimized and act accordingly}
    Label1.Caption := 'Width = ' + IntToStr( R.Right-R.Left);
    Label2.Caption := 'Height = ' + IntToStr( R.Bottom-R.Top);
    Label3.Caption := 'Left = ' + IntToStr( R.Left);
    Label4.Caption := 'Top = ' + IntToStr( R.Top);
   
    if IsIconic(Application.Handle) then
      {minimized - write that state}
      State := Ord(wsMinimized)
    else
      {not mimimized - we can rely on window state of form}
      State := Ord(Self.WindowState);
    Reg.WriteInteger('State', State);


  finally
    Reg.Free;
  end;
end;


procedure TfrmSizePosition.FormShow(Sender: TObject);
var
  Reg: TRegistry;   // the registry
  State: Integer;   // state of window
begin
  Reg := TRegistry.Create;

  try
    // Open required key - and exit it if it doesn't exist
    Reg.RootKey := HKEY_CURRENT_USER;

    if not Reg.OpenKey(CRegKey, False) then Exit;

    // Read the window size and position - designed form sizes are defaults

    Self.Width  := ReadIntFromReg(Reg, 'Width',  Self.Width);
    Self.Height := ReadIntFromReg(Reg, 'Height', Self.Height);
    Self.Left   := ReadIntFromReg(Reg, 'Left',   Self.Left);
    Self.Top    := ReadIntFromReg(Reg, 'Top',    Self.Top);

    // Now get window state and restore
    State := ReadIntFromReg(Reg, 'State', Ord(wsNormal));

    {check if window was minimised
    - we have special processing for minimized state since Delphi doesn't
    minimize windows - it uses application window instead}

  if State = Ord(wsMinimized) then
    begin
      {we need to set visible true else form won't restore properly
      - but this causes a brief display of form}
      Self.Visible := True;
      Application.Minimize;
    end
  else
    Self.WindowState := TWindowState(State);
  finally
    Reg.Free;
  end;


end;

end.
